//
// headers for fit routine testing...
//

#include "utils.h"
#include "errors.h"
#include "getopt.h"
#include <fstream>
#include <cfloat>        // for DBL_EPSILON

cint    maxword = 100;   // max word length
cint    maxline = 200;   // max line length in datafile
ucint    maxlen  = 500;
cint    NPAR = 50;       // default value of npar
cint    MAXITER = 30;    // default value of maxiter 
cdbl    version = 1.5;
cdbl    CONF = 0.68269;  // default value of conf
cdbl    LAMBDA = 1.0e-8; // default value of lambda
cdbl    GAMCR  = 45.0;   // default critical gamma angle

// typedef const string cstr;

enum graphics    {goplot=1, gnuplot, gnups};      // output options
enum off_on      {off, on};
enum no_yes      {no, yes};
enum fail_pass   {fail, pass};

struct flags {
    unsigned
    o,                // yes if output file wanted
    p,                // yes if precision supplied
    q,                // quiet... suppress intermediate output
    s,                // "absolute" sigmas assumed
    et,               // pass if eps_test passed
    get;              // pass if gam_eps_test passed
};

typedef struct {           // miscellaneous quantities used
    string name[NPAR];     // arrays of parameter names
    string ofile[2];       // output file names
    string title;          // a phrase describing the function
    ofstream ofs;          // output file stream used in mk_outfile
    int prec,              // value of ouput precision for params etc
        npar,              // number of parameters
        iter_no,           // iteration number
        maxiter;           // maximum number of iterations allowed
    dbl lambda,            // marquardt parameter
        gamma,             // angle between scaled db and g vectors
        phi,               // sum of squares of residuals
        fract,             // fraction by which curve overlaps data
        conf;              // confidence level
    dbl* lm;               // xmin, xmax, ymin, ymax
} misc;

// prototypes

void calc_db_gamma(dbl*, dbl**, dbl*, misc*);
string chop_ext(const string&);
void copy_mat(dbl**, dbl**, int);
void copy_vec(dbl*, dbl*, int);
dbl del_f(dbl*, dbl*);
int eps_test(dbl*, dbl*, dbl*, int);
dbl fish(dbl, int, int);
dbl fisher(int, int, dbl);
dbl func(dbl*, dbl*, dbl*, dbl*, misc*, int, int);
void func_pts(dbl*, dbl*, dbl*, int, misc*);
int gam_eps_test(dbl*, dbl*, dbl*, dbl*, int);
int gam_lam_test(dbl, dbl);
dbl gamma(dbl*, dbl*, int);
void get_data(char**, dbl*, dbl*, dbl*);
void get_func_info(misc*);
int get_npts(char**);
void get_opts(int, char**, flags*, misc*);
void get_params(char**, dbl*, misc*);
int gjr(dbl**, int);
void mk_outfile(dbl*, dbl*, dbl*, dbl*, misc*, flags*, int);
void new_a(dbl*, dbl*, dbl*, dbl*, dbl*, dbl*, dbl**, misc*, int);
void new_b(dbl*, dbl*, dbl*, int);
dbl norm(int);
dbl normf(int, int);
void output(dbl*, dbl**, misc*, flags*, int, int);
void prn_info(char**);
void prn_usage(char**);
void resid(dbl*, dbl*, dbl*, dbl*, int);
void scale(dbl**, dbl*, dbl*, int);
dbl student(int, dbl);
dbl stud(dbl, int);
dbl sumsq(dbl*, dbl*, int);

