//
// prints output information on the screen
//

#include "fit.h"

void output(dbl* b, dbl** a, misc* mp, flags* fp, int npts, int j)
{
    array ar;
    int k,                          // counter
//        p = mp->prec,               // number of sig figs
        nu = npts - mp->npar;       // degrees of freedom
    unsigned len = 0;               // string length of name[]
    dbl t;                          // student t-factor
    dbl ff;                         // fisher factor
    dbl alpha;                      // significance level
    dbl* se = ar.alloc_d(mp->npar); // standard error array
    dbl* sf = ar.alloc_d(mp->npar); // scale factor array
    dbl* g  = ar.alloc_d(mp->npar); // gradient vector array

    const ios::fmtflags cout_def_flgs = cout.flags(); // save cout flags
    ios::fmtflags flgs = cout_def_flgs  // set new flags
        |ios::right
        |ios::floatfield
        |ios::showpoint;
    cout.flags(flgs);
    cout.precision(mp->prec);
    
// determine length of the max length "name" string

    for (k = 0 ; k < mp->npar ; k++) {
        len = mp->name[k].length() > len ?
        mp->name[k].length() : len;
    }

    switch (j) {
        case 0:                    // starting info
            cout << " ============================\n";
            cout << "   initial parameter values:\n\n";
            for (k = 0 ; k < mp->npar ; k++) {
                cout << setw(len+4) << mp->name[k].c_str()
                    << " = b[" << k << "] = "
                    << b[k] << '\n';
            }
            if ( fp->s ) {         // sigma flag
                cout << "\n   initial chi-sq/df = "
                    << mp->phi/(double)(nu) << "\n";
                cout << "   (we assume \"absolute\" estimates "
                        "of sigmas are provided)" << "\n";
            }
            else {
                cout << "\n   initial phi = "
                    << mp->phi << "\n";
            }
            break;

        case 1:                    // intermediate output
            cout << " ============================\n";
            if (mp->iter_no == 1)
                cout << "  after " << mp->iter_no << " iteration: ";
            else
                cout << "  after " << mp->iter_no << " iterations: ";
            if ( fp->s )
                cout << "chi-sq/df = "
                    << mp->phi/(double)(nu) << " ";
            else
                cout << " phi = " <<  mp->phi << " ";
            cout << "gamma = " << mp->gamma << " ";
            cout << "lambda = " << mp->lambda << "";
            ar.pr_vec("  parameters", b, mp->prec, mp->npar, cout);
            break ;

        case 2:                    // gamma-lambda test bad news

            err("it's bad news.  The gamma-lambda test has\n"
              "revealed, after ",mp->iter_no," iterations, that there may be\n"
              "severe difficulties involving high correlations\n"
              "among the parameters.  You might try reducing\n"
              "the number of parameters so that those remaining\n"
              "are less correlated.\n"
              "\n"
              "p.s. If you see this message displayed on your\n"
              "terminal screen, please send email to drip@cats.\n"
              "we do not know whether this test actually works.\n"
              "it may well be a useless piece of software.\n");

        case 3:                                // passed eps_test (normal)
        case 4:                                // passed gamma_eps_test
            mp->iter_no-- ;                    // passed on previous iter_no
            cout << " ============================\n\n";
            if ( fp->get ) {
                cout <<
                "   Good.  Passing of the gamma-epsilon test "
                "indicates that\n"
                "   after " << mp->iter_no <<
                "   iterations, convergence has been achieved, with\n";
            }
            else {
                cout <<
                "   Good.  Passing of the epsilon test indicates that after\n   "
                  << mp->iter_no <<
                " iterations, normal convergence has been achieved, with\n";
            }

            if ( fp->s ) {                     // sigma flag
                alpha = gammq(0.5 * nu, 0.5 * mp->phi);
                cout <<
                "   a final value of chi-square equal to "
                     << mp->phi << ", calculated\n"
                "   for "
                     << nu << " degrees of freedom.\n\n"
                "   The value of alpha, that is, the probability that a\n"
                "   larger value of chi-square would be obtained for an\n"
                "   equivalent data set, assuming your model to be a good\n"
                "   one, is equal to "
                    << setprecision(2) << alpha
                    << setprecision(mp->prec) << "\n";
            }
            else {
                cout <<
                "   a final value of phi equal to " << mp->phi <<
                ", calculated for\n"
                "   " << nu << " degrees of freedom.\n\n";
            }
            cout <<
               "\n"
               "   Here are the final values of the parameters, along with\n"
               "   estimates of one-parameter uncertainties at the " <<
                   100.*mp->conf << "%\n"
               "   confidence level:\n\n";

            t = student(nu, mp->conf);
            k = gjr(a, mp->npar);   // invert "a" matrix to get se[]
            gassert(k == 0);        // make sure a is non-singular
            for (k = 0 ; k < mp->npar ; k++) {
                if (fp->s)
                    se[k] = a[k][k];
                else
                    se[k] = a[k][k] * mp->phi/(double)(nu);
                gassert(se[k] >= 0.0);
                se[k] = sqrt(se[k]) * t;
                cout << setw(len+4) << mp->name[k].c_str()
                     << " = b[" << k << "] = " << b[k]
                     << " +/- " << se[k] << "\n";
            }
            if (!fp->q) {
                cout << "\n"
                  "   The support plane boundaries for a\n"
                  "   confidence level of " << 100.*mp->conf <<
                  "% are given by:\n\n";
                ff = fisher(mp->npar, npts, mp->conf);
                for (k = 0 ; k < mp->npar ; k++) {
                    se[k] *= sqrt(mp->npar * ff)/t;
                    cout << setw(len+4) << mp->name[k].c_str()
                         << " = b[" << k << "] = " << b[k]
                         << " +/- " << se[k] << "\n";
                }
                cout << "\n";
                scale(a, g, sf, mp->npar);
                cout <<
                      "   Here is the lower half of the (symmetric)\n"
                      "   parameter correlation matrix:";
                ar.pr_sym_mat("", a, mp->prec, mp->npar, cout);
            }
            else {
                cout << "\n";
            }
            if ( fp->o == goplot ) {
                cout << " ===========================================\n";
                cout <<
                    "   Now creating an output file called \"" <<
                    mp->ofile[0] <<
                    "\"\n   for viewing with goplot...\n";
                cout << " ===========================================\n";
            }
            if ( fp->o == gnuplot ) {
                cout << " ===========================================\n";
                cout <<
                    "   Now creating output files called \"" <<
                    mp->ofile[0] <<
                    "\"\n   and \"" <<
                    mp->ofile[1] <<
                    "\" for viewing with gnuplot,\n"
                    "   so try \'gnuplot " << mp->ofile[1] << "\'...\n";
                cout << " ===========================================\n";
            }
            if ( fp->o == gnups ) {
                cout << " ===========================================\n";
                cout <<
                    "   Now creating output files called \"" <<
                    mp->ofile[0] <<
                    "\" and\n   \"" << 
                    mp->ofile[1] <<
                    "\" for making a PostScript file with gnuplot,\n"
                    "   so try \'gnuplot " << mp->ofile[1] << "\'...\n";
                cout << " ===========================================\n";
            }
            cout << "\n";
            break ;
        case 5:                                /* reached max iterations */
            cout << "\n ============================\n\n";
            cout <<
               "   Alas, we have reached the maximum number of " <<
                   mp->iter_no << "\n"
               "   iterations without obtaining convergence.  At\n"
               "   this point, we find ";
            if ( fp->s )
                cout << "  chi-sq/df = " << mp->phi/(double)(nu);
            else
                cout << "  phi = " << mp->phi;
            cout << ",\n   gamma = " << mp->gamma << " degrees,";
            cout << " and lambda = " << mp->lambda << ", with\n";
            ar.pr_vec("   parameters", b, mp->prec, mp->npar, cout);
            break ;
        default:
            err("unidentified option in function output()");
    }
    ar.dele(se);
    ar.dele(g);
    ar.dele(sf);
}
